4.9-stable review patch.  If anyone has any objections, please let me know.

------------------

From: Eric Sandeen <sand...@sandeen.net>

commit 757a69ef6cf2bf839bd4088e5609ddddd663b0c4 upstream.

There are dueling comments in the xfs code about intent
for log writes when unmounting a readonly filesystem.

In xfs_mountfs, we see the intent:

/*
 * Now the log is fully replayed, we can transition to full read-only
 * mode for read-only mounts. This will sync all the metadata and clean
 * the log so that the recovery we just performed does not have to be
 * replayed again on the next mount.
 */

and it calls xfs_quiesce_attr(), but by the time we get to
xfs_log_unmount_write(), it returns early for a RDONLY mount:

 * Don't write out unmount record on read-only mounts.

Because of this, sequential ro mounts of a filesystem with
a dirty log will replay the log each time, which seems odd.

Fix this by writing an unmount record even for RO mounts, as long
as norecovery wasn't specified (don't write a clean log record
if a dirty log may still be there!) and the log device is
writable.

Signed-off-by: Eric Sandeen <sand...@redhat.com>
Reviewed-by: Brian Foster <bfos...@redhat.com>
Reviewed-by: Christoph Hellwig <h...@lst.de>
Reviewed-by: Darrick J. Wong <darrick.w...@oracle.com>
Signed-off-by: Darrick J. Wong <darrick.w...@oracle.com>
Signed-off-by: Greg Kroah-Hartman <gre...@linuxfoundation.org>
---
 fs/xfs/xfs_log.c |    7 +++++--
 1 file changed, 5 insertions(+), 2 deletions(-)

--- a/fs/xfs/xfs_log.c
+++ b/fs/xfs/xfs_log.c
@@ -812,11 +812,14 @@ xfs_log_unmount_write(xfs_mount_t *mp)
        int              error;
 
        /*
         * Don't write out unmount record on read-only mounts.
         * Or, if we are doing a forced umount (typically because of IO errors).
         */
-       if (mp->m_flags & XFS_MOUNT_RDONLY)
+       if (mp->m_flags & XFS_MOUNT_NORECOVERY ||
+           xfs_readonly_buftarg(log->l_mp->m_logdev_targp)) {
+               ASSERT(mp->m_flags & XFS_MOUNT_RDONLY);
                return 0;
+       }
 
        error = _xfs_log_force(mp, XFS_LOG_SYNC, NULL);
        ASSERT(error || !(XLOG_FORCED_SHUTDOWN(log)));
@@ -812,11 +812,14 @@ xfs_log_unmount_write(xfs_mount_t *mp)
 
        /*
-        * Don't write out unmount record on read-only mounts.
+        * Don't write out unmount record on norecovery mounts or ro devices.
         * Or, if we are doing a forced umount (typically because of IO errors).
         */


Reply via email to